@@ -10,12 +10,12 @@ from __future__ import annotations
 """
 
 __app_name__ = "word-search"
-__version__ = "3.0.0"
+__version__ = "3.2.0"
 
 
 import json
 from pathlib import Path
-from typing import Any, Dict, Iterable, List, Set
+from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Set
 
 from . import export, generate, utils
 from .config import (
@@ -27,7 +27,10 @@ from .config import (
     min_puzzle_words,
 )
 from .mask import CompoundMask, Mask
-from .word import Direction, KeyInfo, KeyInfoJson, Wordlist
+from .word import Direction, KeyInfo, KeyInfoJson
+
+if TYPE_CHECKING:  # pragma: no cover
+    from .word import Wordlist
 
 Puzzle = List[List[str]]
 DirectionSet = Set[Direction]
@@ -35,8 +38,20 @@ Key = Dict[str, KeyInfo]
 KeyJson = Dict[str, KeyInfoJson]
 
 
+class PuzzleNotGeneratedError(Exception):
+    """For when a puzzle has yet to be generated."""
+
+    pass
+
+
+class PuzzleSizeError(Exception):
+    """For when the length of each provided word is > `WordSearch.size`."""
+
+    pass
+
+
 class MissingWordError(Exception):
-    """For when a WordSearch cannot include all its words"""
+    """For when a `WordSearch` object cannot place all of its words."""
 
     pass
 
@@ -115,7 +130,7 @@ class WordSearch:
     @property
     def words(self) -> Wordlist:
         """The current puzzle words."""
-        return {word for word in self._words}
+        return set(self._words)
 
     @property
     def placed_words(self) -> Wordlist:
@@ -177,10 +192,7 @@ class WordSearch:
         """The current puzzle state cropped to the mask."""
         min_x, min_y = self.bounding_box[0]
         max_x, max_y = self.bounding_box[1]
-        return [
-            [c for c in row[min_x : max_x + 1]]
-            for row in self.puzzle[min_y : max_y + 1]
-        ]
+        return [list(row[min_x : max_x + 1]) for row in self.puzzle[min_y : max_y + 1]]
 
     @property
     def key(self) -> Key:
@@ -296,12 +308,19 @@ class WordSearch:
     # ************************************************* #
 
     def random_words(
-        self, count: int, secret: bool = False, reset_size: bool = True
+        self,
+        count: int,
+        action: str = "REPLACE",
+        secret: bool = False,
+        reset_size: bool = True,
     ) -> None:
         """Add `count` randomly generated words to the puzzle.
 
         Args:
             count (int): Count of random words to add.
+            action (str): Should the random words be added ("ADD") to the current
+                wordlist or should they replace ("REPLACE") the current wordlist.
+                Defaults to "REPLACE".
             secret (bool, optional): Should the new words
                 be secret. Defaults to False.
             reset_size (bool, optional): Reset the puzzle
@@ -319,12 +338,22 @@ class WordSearch:
                 f"Requested random words must be >= {min_puzzle_words}"
                 + f" and <= {max_puzzle_words}."
             )
-
-        self.add_words(
-            ",".join(utils.get_random_words(count)),
-            secret=secret,
-            reset_size=reset_size,
-        )
+        if not isinstance(action, str):
+            raise TypeError("Action must be a string.")
+        if action.upper() not in ["ADD", "REPLACE"]:
+            raise ValueError("Action must be either 'ADD' or 'REPLACE'.")
+        if action.upper() == "ADD":
+            self.add_words(
+                ",".join(utils.get_random_words(count)),
+                secret=secret,
+                reset_size=reset_size,
+            )
+        else:
+            self.replace_words(
+                ",".join(utils.get_random_words(count)),
+                secret=secret,
+                reset_size=reset_size,
+            )
 
     def show(self, solution: bool = False) -> None:
         """Show the current puzzle with or without the solution.
@@ -357,8 +386,9 @@ class WordSearch:
         """
         if format.upper() not in ["CSV", "JSON", "PDF"]:
             raise ValueError('Save file format must be either "CSV", "JSON", or "PDF".')
-        # validate export path
-        path = export.validate_path(path)
+        # convert strings to PATH object
+        if isinstance(path, str):
+            path = Path(path)
         # write the file
         if format.upper() == "CSV":
             saved_file = export.write_csv_file(path, self)
@@ -382,6 +412,11 @@ class WordSearch:
     def _generate(self, fill_puzzle: bool = True) -> None:
         """Generate the puzzle grid."""
         self._puzzle = utils.build_puzzle(self.size, "")
+        min_word_length = (
+            min([len(word.text) for word in self.words]) if self.words else self.size
+        )
+        if self.size < min_word_length:
+            raise PuzzleSizeError
         for word in self.words:
             word.remove_from_puzzle()
         if not self.mask or len(self.mask) != self.size:
@@ -428,7 +463,8 @@ class WordSearch:
         self._process_input(words, "add", secret)
         if reset_size:
             self.reset_size()
-        self._generate()
+        else:
+            self._generate()
 
     def remove_words(self, words: str, reset_size: bool = False) -> None:
         """Remove words from the puzzle.
@@ -441,7 +477,8 @@ class WordSearch:
         self._process_input(words, "remove")
         if reset_size:
             self.reset_size()
-        self._generate()
+        else:
+            self._generate()
 
     def replace_words(
         self, words: str, secret: bool = False, reset_size: bool = False
@@ -458,7 +495,8 @@ class WordSearch:
         self._process_input(words, "replace", secret)
         if reset_size:
             self.reset_size()
-        self._generate()
+        else:
+            self._generate()
 
     # ************************************************* #
     # ******************** MASKING ******************** #
@@ -466,6 +504,11 @@ class WordSearch:
 
     def apply_mask(self, mask: Mask) -> None:
         """Apply a singular mask object to the puzzle."""
+        if not self.puzzle:
+            raise PuzzleNotGeneratedError(
+                "Puzzle not yet generated. Be sure to add puzzle `words` \
+or set the puzzle `size` before applying a mask."
+            )
         if not isinstance(mask, (Mask, CompoundMask)):
             raise TypeError("Please provide a Mask object.")
         if mask.puzzle_size != self.size:
