@@ -1,4 +1,6 @@
-from word_search_generator.types import Direction
+from __future__ import annotations
+
+from .word import Direction
 
 # puzzle settings
 min_puzzle_size = 5
@@ -7,6 +9,10 @@ min_puzzle_words = 1
 max_puzzle_words = 100
 max_fit_tries = 100
 
+# puzzle grid settings
+ACTIVE = "*"
+INACTIVE = "#"
+
 # puzzle difficulty levels
 level_dirs = {
     1: {Direction.E, Direction.S},
